"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeVerify = void 0;
var errors_1 = require("./errors");
var isPresent = function (item) { return item !== undefined; };
var isBool = function (item) { return typeof item === "boolean"; };
var isNumber = function (item) { return typeof item === "number"; };
var isString = function (item) { return typeof item === "string"; };
var isArray = function (item) { return Array.isArray(item); };
var isTupleArray = function (item) {
    return Array.isArray(item) &&
        item.length > 0 &&
        item.every(function (it) { return isArray(it) && it.length === 2; });
};
var checkGetOptions = function (_a) {
    var options = _a.options;
    if (!isPresent(options))
        return "VerifyResult.NoError" /* NoError */;
    if (isPresent(options.raw) && !isBool(options.raw))
        return "VerifyResult.RawOptionNotBool" /* RawOptionNotBool */;
    if (isPresent(options.retainPath) && !isBool(options.retainPath))
        return "VerifyResult.RetainPathOptionNotBool" /* RetainPathOptionNotBool */;
    if (isPresent(options.max_depth)) {
        if (!isNumber(options.max_depth))
            return "VerifyResult.MaxDepthOptionNotNumber" /* MaxDepthOptionNotNumber */;
        if (options.max_depth < 0)
            return "VerifyResult.MaxDepthOptionIsNegative" /* MaxDepthOptionIsNegative */;
    }
    return "VerifyResult.NoError" /* NoError */;
};
var checkSetValue = function (_a) {
    var path = _a.path, value = _a.value;
    if (isTupleArray(path))
        return "VerifyResult.NoError" /* NoError */;
    if (isArray(path)) {
        if (!isPresent(value) && !isTupleArray(path))
            return "VerifyResult.ValueIsNotTupleArray" /* ValueIsNotTupleArray */;
        if (!isArray(value))
            return "VerifyResult.ValueMustBeArray" /* ValueMustBeArray */;
        if (path.length !== value.length)
            return "VerifyResult.ValueAndPathArraysMustBeSameLength" /* ValueAndPathArraysMustBeSameLength */;
    }
    return value !== undefined && value !== null
        ? "VerifyResult.NoError" /* NoError */
        : "VerifyResult.ValueIsMissing" /* ValueIsMissing */;
};
var checkAddValue = function (_a) {
    var path = _a.path, value = _a.value;
    if (isTupleArray(path))
        return "VerifyResult.NoError" /* NoError */;
    if (isArray(path)) {
        if (!isPresent(value)) {
            if (!isTupleArray(path))
                return "VerifyResult.ValueIsNotTupleArray" /* ValueIsNotTupleArray */;
            else
                return "VerifyResult.NoError" /* NoError */;
        }
        if (!isArray(value))
            return "VerifyResult.ValueMustBeArray" /* ValueMustBeArray */;
        if (path.length !== value.length)
            return "VerifyResult.ValueAndPathArraysMustBeSameLength" /* ValueAndPathArraysMustBeSameLength */;
    }
    return "VerifyResult.NoError" /* NoError */;
};
var checkSetOptions = function (_a) {
    var options = _a.options;
    if (!isPresent(options))
        return "VerifyResult.NoError" /* NoError */;
    if (isPresent(options.raw) && !isBool(options.raw))
        return "VerifyResult.RawOptionNotBool" /* RawOptionNotBool */;
    return "VerifyResult.NoError" /* NoError */;
};
var checkDelOptions = function (_a) {
    var allowPartial = _a.allowPartial;
    if (!isPresent(allowPartial))
        return "VerifyResult.NoError" /* NoError */;
    if (!isBool(allowPartial))
        return "VerifyResult.AllowPartialOptionNotBool" /* AllowPartialOptionNotBool */;
    return "VerifyResult.NoError" /* NoError */;
};
var checkSupportedDMOptions = function (_a) {
    var opts = _a.opts;
    if (!isPresent(opts))
        return "VerifyResult.NoError" /* NoError */;
    var allOptsAreBool = Object.values(opts).every(isBool);
    return allOptsAreBool ? "VerifyResult.NoError" /* NoError */ : "VerifyResult.OptionsMustBeBool" /* OptionsMustBeBool */;
};
var checkInstancesOptions = function (_a) {
    var opts = _a.opts;
    if (!isPresent(opts))
        return "VerifyResult.NoError" /* NoError */;
    if (isPresent(opts.firstLevelOnly) && !isBool(opts.firstLevelOnly))
        return "VerifyResult.FirstLevelOnlyNotBool" /* FirstLevelOnlyNotBool */;
    return "VerifyResult.NoError" /* NoError */;
};
var checkProto = function (_a) {
    var proto = _a.proto;
    if (isPresent(proto) && !isString(proto))
        return "VerifyResult.ProtoNotString" /* ProtoNotString */;
    return "VerifyResult.NoError" /* NoError */;
};
var checkSetPaths = function (_a) {
    var path = _a.path, value = _a.value;
    if (!isPresent(path))
        return "VerifyResult.PathsMissing" /* PathsMissing */;
    if (!isPresent(value) && isArray(path) && !isTupleArray(path))
        return "VerifyResult.ValueIsNotTupleArray" /* ValueIsNotTupleArray */;
    return isString(path) ||
        (isArray(path) &&
            path.length > 0 &&
            path.every(function (path) { return typeof path === "string" || isArray(path); }))
        ? "VerifyResult.NoError" /* NoError */
        : "VerifyResult.PathsNotStringOrStringArray" /* PathsNotStringOrStringArray */;
};
var checkPaths = function (_a) {
    var initalPaths = _a.paths, initialPath = _a.path;
    var paths = initalPaths || initialPath;
    return !isPresent(paths)
        ? "VerifyResult.PathsMissing" /* PathsMissing */
        : isString(paths) ||
            (isArray(paths) &&
                paths.length > 0 &&
                paths.every(function (path) { return typeof path === "string"; }))
            ? "VerifyResult.NoError" /* NoError */
            : "VerifyResult.PathsNotStringOrStringArray" /* PathsNotStringOrStringArray */;
};
var argsVerifiers = {
    GET: [checkPaths, checkGetOptions],
    SET: [checkSetPaths, checkSetValue, checkSetOptions],
    ADD: [checkSetPaths, checkAddValue, checkSetOptions],
    DELETE: [checkPaths, checkDelOptions],
    OPERATE: [checkPaths],
    GET_SUPPORTED_DM: [checkPaths, checkSupportedDMOptions],
    GET_INSTANCES: [checkPaths, checkInstancesOptions],
    GET_SUPPORTED_PROTO: [checkProto],
};
var translateError = function (errors, verifyResult) { return new Error((errors === null || errors === void 0 ? void 0 : errors[verifyResult]) || errors_1.defaultMissingErrorMessage); };
var makeVerify = function (errors) {
    return function (command, args) {
        var verifiers = argsVerifiers[command];
        if (!verifiers)
            return null;
        var results = verifiers.map(function (verifier) { return verifier(args); });
        var finalResult = results.find(function (value) { return value !== "VerifyResult.NoError" /* NoError */; }) ||
            "VerifyResult.NoError" /* NoError */;
        return finalResult === "VerifyResult.NoError" /* NoError */
            ? null
            : translateError(errors, finalResult);
    };
};
exports.makeVerify = makeVerify;
exports.default = exports.makeVerify;
